#!/usr/bin/env python

# servermonitor2.py: Monitor memory, swap, and load on a remote server
# Copyright (C) 2008 Tom Dryer <tomdryer.com@gmail.com> 
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

# A PHP script running on the remote server is downloaded and parsed to gather 
# infomation about free memory, swap, and load average. When these are within 
# defined warning zones, or the server can not be connected too, someone can be 
# notified via email using sSMTP.

# edit options below ##########################################################

# email address to use in the to field
recipient = 'myemail@example.com'

# email address to use in the from field
sender = 'myemail@example.com'

# template for the email message
# first '%s' is recipient, second is sender, third is problem details
message = '''To: %s
From: %s
Subject: servermonitor2.py server alert

One or more tests have failed:
%s'''

# number of minutes to wait between checks
sleeptime = 10

# url of the servermonitor2.php server-side script
url = 'http://example.com/servermonitor2.php'

# warn when there is less than this number of MB of free RAM
free_ram_warning = 50

# warn when there is more than this number of MB of swapped memory
swap_warning = 100

# warn when the 5 minute load average is greater that this number
load_average_warning = 0.75

# options end here ############################################################

import subprocess, urllib, time, re

def send_email(details):
    try:
        ssmtp = subprocess.Popen(('/usr/sbin/ssmtp', recipient), stdin=subprocess.PIPE)
    except OSError:
        print 'could not start sSMTP, email not sent'
    # pass the email contents to sSMTP over stdin
    ssmtp.communicate(message % (recipient, sender, details))
    # wait until the email has finished sending
    ssmtp.wait()
    
def main_loop():
    free_ram_ok = True
    swap_ok = True
    load_average_ok = True
    http_ok = True
    while 1:
        try:
            responsepage = urllib.urlopen(url)
            response = responsepage.read()
        except IOError:
            # could not connect
            http_ok = False
            response = ''
        if response <> '':
            http_ok = True
            free_ram_ok = check_free_ram(response)
            swap_ok = check_swap(response)
            load_average_ok = check_load_average(response)
        if free_ram_ok == True and swap_ok == True and load_average_ok == True and http_ok == True:
            print 'all tests passed'
        else:
            # one or more of the tests have failed
            print 'ALERT: '
            # create a list of tests that failed
            problemlist = ''
            if http_ok == False:
                problemlist = problemlist + 'could not connect\n'
            if free_ram_ok == False:
                problemlist = problemlist + 'free ram warning\n'
            if swap_ok == False:
                problemlist = problemlist + 'swap warning\n'
            if load_average_ok == False:
                problemlist = problemlist + 'load average warning\n'
            print problemlist
            send_email(problemlist)
            print 'attempted to send mail'
        time.sleep(sleeptime*60)
    
def check_free_ram(response):
    ramline = re.compile('buffers/cache: *(\d+) *(\d+)').search(response, 1)
    if ramline == None:
        # the response must not be valid
        return False
    ramvalues = re.compile('\d+').findall(ramline.group(), 1)
    freeram = float(ramvalues[1])
    print str(freeram) + 'MB free'
    if freeram > free_ram_warning:
        return True
    else:
        return False
    
def check_swap(response):
    swapline = re.compile('Swap: *(\d+) *(\d+) *(\d+)').search(response, 1)
    if swapline == None:
        # the response must not be valid
        return False
    swapvalues = re.compile('\d+').findall(swapline.group(), 1)
    usedswap = float(swapvalues[1])
    print str(usedswap) + 'MB swap used'
    if usedswap < swap_warning:
        return True
    else:
        return False

def check_load_average(response):
    loadaverageline =  re.compile('load average: (\d+\.\d+), (\d+\.\d+), (\d+\.\d+)').search(response, 1)
    if loadaverageline == None:
        # the response must not be valid
        return False
    loadaveragevalues = re.compile('\d+\.\d+').findall(loadaverageline.group(), 1)
    # select the 5 min average
    loadaverage = float(loadaveragevalues[1])
    print 'load average is ' + str(loadaverage)
    if loadaverage < load_average_warning:
        return True
    else:
        return False
    
if __name__ == '__main__':
    main_loop()
